//=======================================================================
// DingMee Translator
// URLs: http://chrm.info/cms/dingmee-translator
// Copyright (C) 2012, Christoph Maciejewski
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#ifndef SETTINGSMANAGER_H
#define SETTINGSMANAGER_H

#include <QObject>
#include <QDeclarativeItem>
#include <QStringList>
#include "dictdownloader.h"
#include "common.h"

class SettingsManager : public QDeclarativeItem
{
    Q_OBJECT
public:
    explicit SettingsManager(QDeclarativeItem *parent = 0);

private:
    QMap<int, QString> dictionaryPaths;
    QMap<int, QString> ftpPaths;
    QStringList dictNames;
    QString release;
    QString lastLanguage;

    bool checkForUpdates;
    DictDownloader *dd;

public:
    Q_INVOKABLE void writeSettings();
    Q_INVOKABLE void readSettings();
    Q_INVOKABLE QString getDictPath(int _lang);
    Q_INVOKABLE QString getFTPPath(int _lang);
    Q_INVOKABLE void setDictPath(QString _path, int _lang);
    Q_INVOKABLE void setFTPPath(QString _path, int _lang);
    Q_INVOKABLE void downloadDict(int _lang);
    Q_INVOKABLE void startDownload();
    Q_INVOKABLE QString getStatus();
    Q_INVOKABLE void cancelFtp();
    Q_INVOKABLE QString getDictInfo();
    Q_INVOKABLE bool isDictAvailable(int _dict);
    Q_INVOKABLE bool isDictAvailable();
    Q_INVOKABLE void removeAllDicts();
    Q_INVOKABLE QString getLastLang();

    
signals:
    
public slots:
    
};

#endif // SETTINGSMANAGER_H
