//=======================================================================
// DingMee Translator
// URLs: http://chrm.info/cms/dingmee-translator
// Copyright (C) 2012, Christoph Maciejewski
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "translator.h"
#include <QFile>
#include <QTextStream>
#include <QRegExp>
#include <QSettings>
#include <QDate>

//#define COMPETITION

Translator::Translator(QDeclarativeItem *parent)
    : QDeclarativeItem(parent)
{
    qDebug() << "Translator!";
}

QString Translator::myname() {
    return this->m_name;
}

void Translator::setName(const QString &_name) {
    this->m_name = _name;
}

QString Translator::translate(QString _in, QString _dict) {

    QSettings settings("chrm", "DingMee");
    settings.beginGroup("settings");    
    QFile file(settings.value("dictpath_"+_dict, "/home/user/MyDocs/"+_dict+".txt").toString());
    settings.setValue("lastLanguage", _dict);
    settings.endGroup();

    _in = _in.trimmed();

    qDebug() << "Word: " << _in;

#ifdef COMPETITION
    QDate date(2012, 10, 19);
    int daysLeft = date.daysTo(QDate::currentDate());
    qDebug() << "Days to disable: " << daysLeft;
    if(daysLeft > -1) {
        return "Test version (Maemo.org competition 2012). Please upgrade to the full version!";
    }
#endif

    if(_in.size() < 2) {
        qDebug() << "too short. Exit" << _in;
        return "Enter 2 charactes or more...";
    }

    QStringList langNames = _dict.toUpper().split("-");

    QStringList strings;
    QString tmpString;
    QString retValDE = "";
    QString retValEN = "";   
    QString retVal0 = "";
    QString retVal1 = "";
    QString retVal2 = "";
    QString retValElse = "";
    bool hit0 = false;
    bool hit1 = false;
    bool hit2 = false;
    bool hit3 = false;
    int cnt = 0;

    //QRegExp rx(QString("(.*)("+_in+")(.*)"));
    //rx.setCaseSensitivity(Qt::CaseInsensitive);

    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream in(&file);

        while (!in.atEnd()) {
            tmpString = in.readLine();
            cnt++;

            if(tmpString.startsWith("#"))
                continue;

            if(! tmpString.toLower().contains(_in.toLower())) {
                continue;
            }

            strings = tmpString.split(" :: ");

            if(strings.length() != 2)
                continue;

            retValDE = strings.at(0);
            retValEN = strings.at(1);            

            //retValDE.replace(rx, "\\1<font color='red'>\\2</font>\\3");
            //retValEN.replace(rx, "\\1<font color='red'>\\2</font>\\3");



            if(retValDE.startsWith(_in)) {
                // Exact DE match
                retVal0 += "<tr bgcolor='" + this->getNextColor(0) + "'>";
                retVal0 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal0 += "</tr>";

                hit0 = true;

            } else if(retValEN.startsWith(_in)) {
                // Exact DE match
                retVal0 += "<tr bgcolor='" + this->getNextColor(0) + "'>";
                retVal0 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal0 += "</tr>";

                hit0 = true;

            } else if(retValDE.startsWith(_in + " ") || retValDE.startsWith(_in + ";")) {
                // Exact DE match
                retVal0 += "<tr bgcolor='" + this->getNextColor(0) + "'>";
                retVal0 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal0 += "</tr>";

                hit0 = true;

            } else if(retValEN.startsWith(_in + " ") || retValEN.startsWith(_in + ";")) {
                // Exact DE match
                retVal0 += "<tr bgcolor='" + this->getNextColor(0) + "'>";
                retVal0 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal0 += "</tr>";

                hit0 = true;

            } else if(retValDE.toLower().startsWith(_in.toLower() + " ") || retValDE.toLower().startsWith(_in.toLower() + ";")) {
                // Exact DE match
                retVal1 += "<tr bgcolor='" + this->getNextColor(1) + "'>";
                retVal1 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal1 += "</tr>";

                hit1 = true;

            } else if(retValEN.toLower().startsWith(_in.toLower() + " ") || retValEN.toLower().startsWith(_in.toLower() + ";")) {
                // Exact EN match                
                retVal1 += "<tr bgcolor='" + this->getNextColor(1) + "'>";
                retVal1 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal1 += "</tr>";

                hit1 = true;

            } else if(retValDE.contains(" " + _in + " ") || retValDE.contains(" " + _in + ";")) {
                // Very exact DE match
                retVal2 += "<tr bgcolor='" + this->getNextColor(2) + "'>";
                retVal2 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal2 += "</tr>";

                hit2 = true;

            } else if(retValEN.contains(" " + _in + " ") || retValEN.contains(" " + _in + ";")) {
                // Very exact EN match
                retVal2 += "<tr bgcolor='" + this->getNextColor(2) + "'>";
                retVal2 += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retVal2 += "</tr>";

                hit2 = true;

            } else if(retValDE.toLower().contains(" "+_in.toLower())) {
                // Similar DE match
                if((retVal0 + retVal1 + retVal2 + retValElse).length() > 500) {
                    continue;
                }                
                retValElse += "<tr bgcolor='" + this->getNextColor(3) + "'>";
                retValElse += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retValElse += "</tr>";

                hit3 = true;

            } else if(retValEN.toLower().contains(" "+_in.toLower())) {
                // Similar EN match
                if((retVal0 + retVal1 + retVal2 + retValElse).length() > 500) {
                    continue;
                }
                retValElse += "<tr bgcolor='" + this->getNextColor(3) + "'>";
                retValElse += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retValElse += "</tr>";                

                hit3 = true;

           } else if(retValDE.toLower().startsWith(_in.toLower())) {
                // Exact DE match
                retValElse += "<tr bgcolor='" + this->getNextColor(1) + "'>";
                retValElse += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retValElse += "</tr>";

                hit3 = true;

            } else if(retValEN.toLower().startsWith(_in.toLower())) {
                // Exact EN match
                retValElse += "<tr bgcolor='" + this->getNextColor(1) + "'>";
                retValElse += "<td>" + splitDataset(retValDE) + "</td><td>" + splitDataset(retValEN) + "</td>";
                retValElse += "</tr>";

                hit3 = true;
            }
        }
    } else {
        return "Could not open File: " + file.fileName();
    }

    file.close();

    retVal0.replace(_in, "<font color='red'>"+_in+"</font>");
    retVal1.replace(_in.toLower(), "<font color='red'>"+_in.toLower()+"</font>");
    retVal2.replace(_in, "<font color='red'>"+_in+"</font>");
    //retVal.replace(QRegExp("(.*)("+_in+")(.*)"), "\\1<font color='red'>\\2</font>\\3");
    //retVal.replace(QRegExp("(.*)("+_in.toLower()+")(.*)"), "\\1<font color='red'>\\2</font>\\3");

    retVal0 = "<table border='0' width='100%'><tr><td width='50%'><font color=green><b>"+langNames[0]+"</b></font></td><td width='50%' align='right'><font color=blue><b>"+langNames[1]+"</b></font></td></tr>" + retVal0;
    retVal1 = "<table border='0' width='100%'><tr><td width='50%'><font color=green><b>"+langNames[0]+"</b></font></td><td width='50%' align='right'><font color=blue><b>"+langNames[1]+"</b></font></td></tr>"+ retVal1;
    retVal2 = "<table border='0' width='100%'>"+ retVal2;
    retValElse = "<table border='0' width='100%'>" +retValElse;

    retVal0 += "</table>";
    retVal1 += "</table>";
    retVal2 += "</table>";
    retValElse += "</table>";    


    if(hit0 && hit1 && hit2 && hit3) {        
        return retVal0 + "<br><br><i>More results:</i><br><br>" + retVal1 + "<br><br><i>Much more results:</i><br><br>" + retVal2 + "<br><br><i>More results:</i><br><br>" + retValElse;
    } else if(hit0 && hit1 && hit2) {        
        return retVal0 + "<br><br><i>More results:</i><br><br>" + retVal1 + "<br><br><i>Much more results:</i><br><br>" + retVal2;
    } else if(hit0 && hit1) {        
        return retVal0 + "<br><br><i>More results:</i><br><br>" + retVal1;
    } else if(hit0 && hit2) {
        return retVal0 + "<br><br><i>More results:</i><br><br>" + retVal2;
    } else if(hit1 && hit2) {
        return retVal1 + "<br><br><i>More results:</i><br><br>" + retVal2;
    } else if(hit0) {
        return retVal0;
    } else if(!hit0 && !hit1 && !hit2 && !hit3) {
        return "<i>Sorry, no results.</i>";
    }

    return retVal1 + "<br>" + retVal2 + "<br>" + retValElse;
}

QString Translator::getNextColor(int _type) {
    QString color1 = "#f0f0f0";
    QString color2 = "#cefffc";

    if(this->currentColor[_type] == color1)
        this->currentColor[_type] = color2;
    else
        this->currentColor[_type] = color1;

    return this->currentColor[_type];
}

QString Translator::splitDataset(QString _in) {
    QStringList wordList = _in.split(" | ");
    QString retVal = "<b>" + wordList.at(0) + "</b><br>";

    for(int i=1; i<wordList.length(); i++) {
        retVal += QString::number(i) + ")&nbsp;"+wordList.at(i)+"<br>";
    }

    return retVal;
}
